@@ -359,7 +359,7 @@ bool Application::slurp()
     int error;
     switch ((error= errno))
     {
-#ifdef TARGET_OS_LINUX
+#ifdef __linux
     case ERESTART:
 #endif
     case EINTR:
@@ -388,30 +388,75 @@ bool Application::slurp()
   }
 
   bool data_was_read= false;
-  if (fds[0].revents & POLLRDNORM)
+  if (fds[0].revents)
   {
-    if (stdout_fd.read(_stdout_buffer) == true)
+    if (fds[0].revents & POLLRDNORM)
     {
-      data_was_read= true;
+      if (stdout_fd.read(_stdout_buffer) == true)
+      {
+        data_was_read= true;
+      }
+    }
+
+    if (fds[0].revents & (POLLERR | POLLHUP | POLLNVAL))
+    {
+      stdout_fd.close();
+
+      if (fds[0].revents & POLLERR)
+      {
+        Error << "getsockopt(stdout)";
+      }
     }
   }
 
-  if (fds[1].revents & POLLRDNORM)
+  if (fds[1].revents)
   {
-    if (stderr_fd.read(_stderr_buffer) == true)
+    if (fds[1].revents & POLLRDNORM)
     {
-      data_was_read= true;
+      if (stderr_fd.read(_stderr_buffer) == true)
+      {
+        data_was_read= true;
+      }
+    }
+
+    if (fds[1].revents & (POLLERR | POLLHUP | POLLNVAL))
+    {
+      stderr_fd.close();
+
+      if (fds[1].revents & POLLERR)
+      {
+        Error << "getsockopt(stderr)";
+      }
     }
   }
 
   return data_was_read;
 }
 
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wunreachable-code"
 Application::error_t Application::join()
 {
-  pid_t waited_pid= waitpid(_pid, &_status, 0);
+  if (waitpid(_pid, &_status, 0) == -1)
+  {
+    std::string error_string;
+    if (stdout_result_length())
+    {
+      error_string+= " stdout: ";
+      error_string+= stdout_c_str();
+    }
+
+    if (stderr_result_length())
+    {
+      error_string+= " stderr: ";
+      error_string+= stderr_c_str();
+    }
+    Error << "waitpid() returned errno:" << strerror(errno) << " " << error_string;
+    return Application::UNKNOWN;
+  }
+
   slurp();
-  if (waited_pid == _pid and WIFEXITED(_status) == false)
+  if (WIFEXITED(_status) == false)
   {
     /*
       What we are looking for here is how the exit status happened.
@@ -476,26 +521,9 @@ Application::error_t Application::join()
         << " name:" << built_argv[0];
     }
   }
-  else if (waited_pid == _pid and WIFEXITED(_status))
+  else if (WIFEXITED(_status))
   {
-    _app_exit_state= int_to_error_t(WEXITSTATUS(_status));
-  }
-  else if (waited_pid == -1)
-  {
-    std::string error_string;
-    if (stdout_result_length())
-    {
-      error_string+= " stdout: ";
-      error_string+= stdout_c_str();
-    }
-
-    if (stderr_result_length())
-    {
-      error_string+= " stderr: ";
-      error_string+= stderr_c_str();
-    }
-    Error << "waitpid() returned errno:" << strerror(errno) << " " << error_string;
-    _app_exit_state= Application::UNKNOWN;
+    return int_to_error_t(WEXITSTATUS(_status));
   }
   else
   {
@@ -505,6 +533,7 @@ Application::error_t Application::join()
 
   return _app_exit_state;
 }
+#pragma GCC diagnostic pop
 
 void Application::add_long_option(const std::string& name, const std::string& option_value)
 {
@@ -546,6 +575,25 @@ int Application::Pipe::Pipe::fd()
   return _pipe_fd[WRITE]; // STDIN_FILENO
 }
 
+void Application::Pipe::Pipe::close()
+{
+  if (_std_fd == STDOUT_FILENO)
+  {
+    ::close(_pipe_fd[READ]);
+    _pipe_fd[READ]= -1;
+  }
+  else if (_std_fd == STDERR_FILENO)
+  {
+    ::close(_pipe_fd[READ]);
+    _pipe_fd[READ]= -1;
+  }
+  else
+  {
+    ::close(_pipe_fd[WRITE]);
+    _pipe_fd[WRITE]= -1;
+  }
+}
+
 
 bool Application::Pipe::read(libtest::vchar_t& arg)
 {
@@ -596,7 +644,7 @@ void Application::Pipe::nonblock()
   if (flags == -1)
   {
     Error << "fcntl(F_GETFL) " << strerror(errno);
-    throw strerror(errno);
+    throw std::runtime_error(strerror(errno));
   }
 
   int rval;
@@ -608,7 +656,7 @@ void Application::Pipe::nonblock()
   if (rval == -1)
   {
     Error << "fcntl(F_SETFL) " << strerror(errno);
-    throw strerror(errno);
+    throw std::runtime_error(strerror(errno));
   }
 }
 
@@ -649,7 +697,7 @@ void Application::Pipe::cloexec()
       if (flags == -1)
       {
         Error << "fcntl(F_GETFD) " << strerror(errno);
-        throw strerror(errno);
+        throw std::runtime_error(strerror(errno));
       }
 
       int rval;
@@ -661,7 +709,7 @@ void Application::Pipe::cloexec()
       if (rval == -1)
       {
         Error << "fcntl(F_SETFD) " << strerror(errno);
-        throw strerror(errno);
+        throw std::runtime_error(strerror(errno));
       }
     }
   }
@@ -731,13 +779,26 @@ void Application::create_argv(const char *args[])
     vchar::append(built_argv, "--leak-check=yes");
 #if 0
     vchar::append(built_argv, "--show-reachable=yes"));
-#endif
     vchar::append(built_argv, "--track-fds=yes");
+#endif
 #if 0
     built_argv[x++]= strdup("--track-origin=yes");
 #endif
     vchar::append(built_argv, "--malloc-fill=A5");
     vchar::append(built_argv, "--free-fill=DE");
+    vchar::append(built_argv, "--xml=yes");
+    if (getenv("VALGRIND_HOME"))
+    {
+      libtest::vchar_t buffer;
+      buffer.resize(1024);
+      int length= snprintf(&buffer[0], buffer.size(), "--xml-file=%s/cmd-%%p.xml", getenv("VALGRIND_HOME"));
+      fatal_assert(length > 0 and size_t(length) < buffer.size());
+      vchar::append(built_argv, &buffer[0]);
+    }
+    else
+    {
+      vchar::append(built_argv, "--xml-file=var/tmp/valgrind-cmd-%p.xml");
+    }
 
     std::string log_file= create_tmpfile("valgrind");
     libtest::vchar_t buffer;
