@@ -153,6 +153,66 @@ static test_return_t short_help_test(void *)
   return TEST_SUCCESS;
 }
 
+static test_return_t long_keepalive_TEST(void *)
+{
+  const char *args[]= { "--check-args", "--keepalive", 0 };
+
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  return TEST_SUCCESS;
+}
+
+static test_return_t long_keepalive_idle_TEST(void *)
+{
+  const char *args[]= { "--check-args", "--keepalive-idle=10", 0 };
+
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  return TEST_SUCCESS;
+}
+
+static test_return_t long_keepalive_interval_TEST(void *)
+{
+  const char *args[]= { "--check-args", "--keepalive-interval=3", 0 };
+
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  return TEST_SUCCESS;
+}
+
+static test_return_t long_keepalive_count_TEST(void *)
+{
+  const char *args[]= { "--check-args", "--keepalive-count=10", 0 };
+
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  return TEST_SUCCESS;
+}
+
+static test_return_t long_keepalive_start_TEST(void *)
+{
+  in_port_t port= libtest::get_free_port();
+  char port_str[1024];
+  test_true(snprintf(port_str, sizeof(port_str), "--port=%d", int32_t(port)) > 0);
+
+  const char *args[]= {
+    port_str,
+    "--check-args",
+    "--log-file=stderr",
+    "--keepalive-count=10", 
+    "--keepalive-interval=3",
+    "--keepalive-idle=10", 0 };
+
+  Application app(gearmand_binary(), true);
+
+  Application::error_t ret= app.run(args);
+
+  ASSERT_EQ(Application::SUCCESS, ret);
+
+  ret= app.join();
+  ASSERT_EQ_(Application::SUCCESS, ret, "ret: %s stderr: '%s' stdout: `%s'",
+             Application::toString(ret),
+             app.stderr_c_str(), app.stdout_c_str());
+
+  return TEST_SUCCESS;
+}
+
 static test_return_t long_log_file_test(void *)
 {
   const char *args[]= { "--check-args", "--log-file=\"tmp/foo\"", 0 };
@@ -249,6 +309,14 @@ static test_return_t long_syslog_test(void *)
   return TEST_SUCCESS;
 }
 
+static test_return_t SSL_TEST(void *)
+{
+  const char *args[]= { "--check-args", "--ssl", 0 };
+
+  ASSERT_EQ(EXIT_SUCCESS, exec_cmdline(gearmand_binary(), args, true));
+  return TEST_SUCCESS;
+}
+
 static test_return_t long_threads_test(void *)
 {
   const char *args[]= { "--check-args", "--threads=10", 0 };
@@ -529,7 +597,7 @@ static test_return_t maxqueue_TEST(void *)
 
   ASSERT_EQ(Application::SUCCESS, gearmand.run());
 
-  Worker worker(port);
+  test::Worker worker(port);
   ASSERT_EQ(GEARMAN_SUCCESS, gearman_worker_register(&worker, __func__, 0));
   ASSERT_EQ(GEARMAN_SUCCESS, gearman_worker_unregister(&worker, __func__));
   ASSERT_EQ(Application::SUCCESS, gearmand.join());
@@ -555,6 +623,10 @@ test_st gearmand_option_tests[] ={
   {"-f", 0, short_file_descriptors_test},
   {"--help", 0, long_help_test},
   {"-h", 0, short_help_test},
+  {"--keepalive", 0, long_keepalive_TEST},
+  {"--keepalive-idle", 0, long_keepalive_idle_TEST},
+  {"--keepalive-interval", 0, long_keepalive_interval_TEST},
+  {"--keepalive-count", 0, long_keepalive_count_TEST},
   {"--log-file=", 0, long_log_file_test},
   {"--log-file=stderr", 0, long_log_file_stderr_TEST},
   {"-l", 0, short_log_file_test},
@@ -566,6 +638,7 @@ test_st gearmand_option_tests[] ={
   {"-P", 0, short_pid_file_test},
   {"--round-robin", 0, long_round_robin_test},
   {"-R", 0, short_round_robin_test},
+  {"--ssl", 0, SSL_TEST},
   {"--syslog=", 0, long_syslog_test},
   {"--threads=", 0, long_threads_test},
   {"-T", 0, short_threads_test},
@@ -597,6 +670,7 @@ test_st gearmand_option_tests[] ={
   {"--config-file", 0, config_file_DEFAULT_TEST },
   {"--config-file=etc/grmandfoo.conf", 0, config_file_FAIL_TEST },
   {"--config-file=etc/gearmand.conf", 0, config_file_SIMPLE_TEST },
+  {"start server with all --keepalive options", 0, long_keepalive_start_TEST},
   {0, 0, 0}
 };
 
