@@ -2,7 +2,7 @@
  * 
  *  Gearmand client and server library.
  *
- *  Copyright (C) 2011-2012 Data Differential, http://datadifferential.com/
+ *  Copyright (C) 2011-2013 Data Differential, http://datadifferential.com/
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -165,16 +165,9 @@ static void thread_runner(context_st* con)
     gearman_worker_set_namespace(&worker, context->namespace_key.c_str(), context->namespace_key.length());
   }
 
-  // Check for a working server by "asking" it for an option
+  // Set worker id
   {
-    size_t count= 5;
-    bool success= false;
-    while (--count and success == false)
-    {
-      success= gearman_worker_set_server_option(&worker, test_literal_param("exceptions"));
-    }
-
-    if (success == false)
+    if (gearman_failed(gearman_worker_set_identifier(&worker, gearman_literal_param("start_worker"))))
     {
       Out << "gearman_worker_set_server_option() failed";
       context->fail();
@@ -182,6 +175,14 @@ static void thread_runner(context_st* con)
     }
   }
 
+  // Check for a working server by pinging it with echo
+  if (gearman_failed(gearman_worker_echo(&worker, gearman_literal_param("start_worker"))))
+  {
+    Out << "gearman_worker_set_server_option() failed";
+    context->fail();
+    return;
+  }
+
   if (gearman_failed(gearman_worker_define_function(&worker,
                                                     context->function_name.c_str(), context->function_name.length(),
                                                     context->worker_fn,
@@ -213,15 +214,17 @@ static void thread_runner(context_st* con)
 
     if (ret == GEARMAN_SHUTDOWN)
     {
-      if (gearman_failed(gearman_worker_unregister_all(&worker)))
+      gearman_return_t unreg_ret;
+      if (gearman_failed((unreg_ret= gearman_worker_unregister_all(&worker))))
       {
-        Error << "Failed to unregister " << context->function_name;
+        Error << "Failed to unregister " << context->function_name << " " << gearman_strerror(unreg_ret);
       }
       continue;
     }
 
     if (ret != GEARMAN_SUCCESS and ret != GEARMAN_INVALID_ARGUMENT and ret != GEARMAN_WORK_FAIL)
     {
+      context->handle->error();
 #if 0
       Error <<  context->function_name << ": " << gearman_strerror(ret) << ": " << gearman_worker_error(&worker);
 #endif
@@ -253,7 +256,7 @@ worker_handle_st *test_worker_start(in_port_t port,
     delete context;
     delete handle;
 
-    return NULL;
+    FATAL("Could not allocate worker");
   }
 
   handle->wait();
@@ -274,6 +277,7 @@ void worker_handle_st::set_worker_id(gearman_worker_st* worker)
 worker_handle_st::worker_handle_st() :
   failed_startup(false),
   _shutdown(false),
+  _error_count(0),
   _worker_id(gearman_id_t()),
   _sync_point(2)
 {
@@ -325,7 +329,7 @@ bool worker_handle_st::shutdown()
 bool worker_handle_st::check()
 {
   gearman_return_t rc;
-  if (gearman_failed(rc=  gearman_kill(_worker_id, GEARMAN_KILL)))
+  if (gearman_failed(rc=  gearman_kill(_worker_id, GEARMAN_SIGNAL_CHECK)))
   {
     return false;
   }
