@@ -199,7 +199,7 @@ static void *world_create(server_startup_st& servers, test_return_t&)
   SKIP_IF(HAVE_TOKYOCABINET != 1);
 
   unlink("var/tmp/gearman.tcb");
-  return new Context(libtest::default_port(), servers);
+  return new Context(servers);
 }
 
 static bool world_destroy(void *object)
